﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.IO.Ports;
using System.Windows.Threading;
using LiveCharts;
using LiveCharts.Wpf;
using LiveCharts.Defaults;

namespace Brew_Logic
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    
    public partial class MainWindow : Window
    {
        DispatcherTimer startBrew = new DispatcherTimer();
        SerialPort myport = new SerialPort();
        bool stopTemp = false;
        
        

        public SeriesCollection SeriesCollection { get; private set; }

        public MainWindow()
        {
            InitializeComponent();

            SetOutputLimits(0, 255);
            SetTunings(100, 1, 1, 1);
            SetMode(0);
            SetSampleTime(1000);
            Setpoint = Convert.ToDouble(setPointTxt.Text);

                                 

            //Initialize();
            
            startBrew.Interval = TimeSpan.FromSeconds(1);
            //startBrew.Tick += new EventHandler(dispatcherTimer_Tick);
            cartesianChart1.AxisY.Add(
        new Axis
        {
            MinValue = 0,MaxValue = 100
        });

            SeriesCollection = new SeriesCollection
                {
                new LineSeries
                {
                    Values = new ChartValues<double> {  }
                },
                new LineSeries
                {
                    Values = new ChartValues<double> {}
                }
                
            };

                     
            DataContext = this;
            //SerialPort myport = new SerialPort();
            myport.BaudRate = 9600;
            myport.PortName = "COM4";
            try
            {
                myport.Open();
            }
            catch
            {
               // MessageBox.Show("nothing connected");
            }
            
            
            /*
            while (true)
            {
                string potTemp = myport.ReadLine();
                Console.WriteLine(potTemp);
            }
            */
        }

       

        public ulong lastTime;
        public double Input, Output, Setpoint;
        public double outputSum, lastInput;
        public double kp, ki, kd;
        int SampleTime = 1000; //1 sec
        double outMin, outMax;
        public bool inAuto = false;

        public int controllerDirection = 0;
        bool pOnE = true, pOnM = false;
        public double pOnEKp, pOnMKp;

        public void Compute()
        {
            if (!inAuto) return;
            //How Long since the last calcuated value
            ulong now = Convert.ToUInt64( DateTime.Now.Millisecond);
            ulong timeChange = (now + lastTime);
            if (Convert.ToInt64(timeChange) > SampleTime)
            {

                //Compute all he working error variables
                double error = Setpoint - Input;
                double dInput = (Input - lastInput);
                outputSum += (ki * error);

                /*Add Proportional on Measurement, if P_ON_M is specified*/
                if (pOnM) outputSum -= pOnMKp * dInput;


                if (outputSum > outMax)
                {
                    outputSum = outMax;
                }
                else if (outputSum < outMin)
                {
                    outputSum = outMin;
                }

                /*Add Proportional on Error, if P_ON_E is specified*/
                if (pOnE) Output = pOnEKp * error;
                else Output = 0;


                //Compute PID Output
                Output += outputSum - kd * dInput;

                if (Output > outMax)
                {
                    Output = outMax;
                }
                else if (Output < outMin)
                {
                    Output = outMin;
                }
            }
                //Remember some variables for next time
                lastInput = Input;
                lastTime = now;

            }
        

        public void SetTunings(double Kp, double Ki, double Kd, double pOn)
        {
            if (Kp < 0 || Ki < 0 || Kd < 0 || pOn < 0 || pOn > 1) return;

            pOnE = pOn > 0; // some p on error is desired;
            pOnM = pOn < 1; //some p on measurement is desired;


            double SampleTimeInSec = (Convert.ToDouble(SampleTime)) / 1000;
            kp = Kp;
            ki = Ki * SampleTimeInSec;
            kd = Kd / SampleTimeInSec;

            if(controllerDirection == 1)
            {
                kp = (0 - kp);
                ki = (0 - ki);
                kd = (0 - kd);
            }

            pOnEKp = pOn * kp;
            pOnMKp = (1 - pOn) * kp;
        }

        public void SetSampleTime (int NewSampleTime)
        {
            if (NewSampleTime >0)
            {
                double ratio = Convert.ToDouble(NewSampleTime) / Convert.ToDouble(SampleTime);
                ki *= ratio;
                kd /= ratio;
                SampleTime = NewSampleTime;
            }
        }

        public void SetOutputLimits (double Min, double Max)
        {
            if (Min>Max)
            {
                return;
            }
            outMin = Min;
            outMax = Max;

            if(Output > outMax)
            {
                Output = outMax;
            }
            else if(Output < outMin)
            {
                Output = outMin;
            }
            
            if(outputSum > outMax)
            {
                outputSum = outMax;
            }
            else if (outputSum < outMin)
            {
                outputSum = outMin;
            }
        }
        
        public void SetMode (int Mode)
        {
            bool newAuto = (Mode == 1);
            if(newAuto && !inAuto)
            { //we just went from manual to auto
               InitializeMode();
                ledSlider.IsEnabled = false;
            }
            inAuto = newAuto;
            
        }

        public void InitializeMode()
        {
            lastInput = Input;
            outputSum = Output;
            if (outputSum > outMax) outputSum = outMax;
            else if (outputSum < outMin) outputSum = outMin;
        }

        public void SetControllerDirection(int Direction)
        {
            controllerDirection = Direction;
        }



        private void ckbManualClick(object sender, RoutedEventArgs e)
        {
            ledSlider.IsEnabled = true;
            SetMode(0);
        }

        private void ckbAutoClick(object sender, RoutedEventArgs e)
        {
            ledSlider.IsEnabled = false;
            SetMode(1);
        }


        private void ledSliderChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            var slider = sender as Slider;
            double value = slider.Value;
            string serialInstruction = string.Format("<4,{0}>", value);
            myport.Write(serialInstruction);
            //double currentPercent = value / 255 * 100;
            //txtTest.Text = currentPercent.ToString();

        }

        private void btnPumpClick(object sender, RoutedEventArgs e)
        {
            myport.Write("<1>");
        }

        private void btnFanClick(object sender, RoutedEventArgs e)
        {
            myport.Write("<2>");
        }

        private void getTempClick(object sender, RoutedEventArgs e)
        {
            
            
            
            //dispatcherTimer.Stop();
            if (stopTemp == false)
            {
                startBrew.Tick += new EventHandler(dispatcherTimer_Tick);
                stopTemp = true;
                startBrew.Start();
                
            }
            else
            {
                stopTemp = false;
                startBrew.Stop();
                startBrew.Interval = new TimeSpan(0, 0, 1);
                startBrew.Tick -= new EventHandler(dispatcherTimer_Tick);
            }
            
        }

        private void dispatcherTimer_Tick(object sender, EventArgs e)
        {

            myport.Write("<3>");
            string potTemp = myport.ReadLine();
            currentTemp.Text = potTemp;
            potTemp = potTemp.Replace("\r", "");
            Setpoint = Convert.ToInt32(setPointTxt.Text);
            Input = Convert.ToDouble(potTemp);
            Compute(); // starts pid calculation

            
            ledSlider.Value = Output;
            string serialInstruction = string.Format("<4,{0}>", Output);
            myport.Write(serialInstruction);
            //lsbTemp.Items.Add(Output);
            //lsbTemp.Items.Add(DateTime.Now.ToString("HH:mm:ss"));
                        
            //double intensity = ledSlider.Value;
            //double currentIntensity = intensity / 255 * 100;
            
            

            //SeriesCollection[0].Values.Add(double.Parse(potTemp));
            //SeriesCollection[1].Values.Add(currentIntensity);
            
        }
    }
}